#include <stdint.h>
#include "display.h"
#include "uart.h"

/*
   NOTE:
   
   RTS/CTS handshake must be used for communication with the module.
   These signals provide synchronization between MCU and WF121.

*/


/** Uncomment to debug the module UART channel */
//#define MSG_DEBUG

/** UART hardware flow control pins (CTS/RTS) */
sbit MCU_CTS at GPIOD_IDR.B13;
sbit MCU_RTS at GPIOD_ODR.B10;

/** UART recieve buffer */
uint8_t rx_buffer;



void init_WF121_UART()
{
    /** Set RTS/CTS pins */
    GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_10);
    GPIO_Digital_Input(&GPIOD_BASE, _GPIO_PINMASK_13);
    
    /** Tell module not to send data */
    MCU_RTS=0;
    delay_ms(100);
    MCU_RTS=1;

    /** UART0 channel for BGAPI */
    UART3_Init_Advanced(115200, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART3_PD89);
    show_status("UART3 initialized",1);


    //Message stays if no module is connected
    show_status("Please connect the module to mikroBUS 1",0);
    show_status("and restart the MCU.",0);
}

/** Sends data to the module */
int8_t uart_send (uint16_t data_len, uint8_t* _data)
{
        uint8_t* data_ptr = _data;
        #ifdef MSG_DEBUG
        show_status("Transmitting data...",1);
        #endif
        if (data_len <= 0)
            return -1;
          while(data_len)
          {
              if( MCU_CTS == 0 )
              {
                  #ifdef MSG_DEBUG
                  show_status_hex(*(data_ptr),3);
                  #endif

                  UART3_Write(*(data_ptr++));

                  data_len--;
              }
          }
        #ifdef MSG_DEBUG
        show_status("Data transmitted!",1);
        #endif
        return 0;
}

/** Reads data from the module*/
int8_t uart_get (uint16_t data_len, uint8_t *buff)
{
        uint8_t count = data_len;
        if (data_len <= 0)
           return -1;
        #ifdef MSG_DEBUG
        show_status("Receiving data...",1);
        #endif
        while(count--)
          {
             MCU_RTS = 0;
             while(!UART3_Data_Ready());
             MCU_RTS = 1;
             
             rx_buffer = UART3_Read();

             *(buff++) = rx_buffer;
             #ifdef MSG_DEBUG
             show_status_hex(rx_buffer,3);
             #endif
          }
        #ifdef MSG_DEBUG
        show_status("Data received!",1);
        #endif
        return 0;
}